package oliot.aika.olio;

/**
 * Ensimmäinen kunnon olioesimerkki
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);    a1.toString() === "13:10";
 * </pre>
 */
public class Aika {
    // ei gloaableja
    private int h;
    private int m;

    public final int aseta(int ih,int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit väille -59 - 59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit välille 0-23

        if (tm<0) { tm += 60; th--; }  //negatiiviset arvot
        if (th<0) { th += 24; vrk--; }

        this.h = th; // asetetaan lasketut arvot attribuutteihin
        this.m = tm;
        return vrk; // motako vuorokautta jäi yli    
    }    

    public Aika() { // oletusmuodostaja
       // h = 0;
       // m = 0;
       // this(0,0);
    }
    
    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h, int m) { // Muodostaja
        aseta(h, m);
    }


    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", h, m);
    }


    /**
     * Lisää aikaan valitun minuuttimäärän
     * @param lisaMin lisättävä minuuttimäärä
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    public void lisaa(int lisaMin) {
        int yht_min = this.h * 60 + m + lisaMin;
        h = yht_min / 60;
        m = yht_min % 60;
    }
    
    public int getH() { return h; }
    public int getM() { return m; }
    
    public void lisaa() {
        lisaa(1);
    }
    
}