package oliot.aika.rajapinta;

/**
 * Rajapainta kaikille aikaluokille
 * @author vesal
 * @version 15.2.2022
 *
 */
public interface AikaRajapinta {

    /**
     * Asettaa uuden ajan ja pitää huolen että aika on aina oikeaa muotoa.
     * @param ih asetettavat tunnit
     * @param im asetettavat minuutit
     * @return yli- tai alijääneet vuorokaudet
     * @example
     * <pre name="test">
     *   Aika a = new Aika();
     *   a.aseta(25,12)  ===  1;  a.toString() === "01:12";
     *   a.aseta(3,90)   ===  0;  a.toString() === "04:30";
     *   a.aseta(3,-10)  ===  0;  a.toString() === "02:50";
     *   a.aseta(-3,-10) === -1;  a.toString() === "20:50";
     *   a.aseta(3,-70)  ===  0;  a.toString() === "01:50";
     * </pre>
     */
    int aseta(int ih, int im);


    /**
     * Asettaa uuden ajan ja pitää huolen että aika on aina oikeaa muotoa.
     * @param ih asetettavat tunnit
     * @return yli- tai alijääneet vuorokaudet
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,30);
     *   a.aseta(25)  ===  1;  a.toString() === "01:00";
     *   a.aseta(3)   ===  0;  a.toString() === "03:00";
     *   a.aseta(-3)  === -1;  a.toString() === "21:00";
     * </pre>
     */
    int aseta(int ih);


    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * Aika a = new Aika(12,5);
     * a.toString() === "12:05";
     * </pre>
     */
    @Override
    String toString();


    /**
     * Lisää aikaan valitun minuuttimäärän
     * @param lisaMin lisättävä minuuttimäärä
     * @return ylimenneet vuorokaudet
     * @example
     * <pre name="test">
     * Aika a = new Aika(12,50);
     * a.lisaa(30) === 0; a.toString() === "13:20";
     * a.lisaa(12*60) === 1; a.toString() === "01:20";
     * </pre>
     */
    int lisaa(int lisaMin);


    /**
     * @return ajan tunnit
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,50);
     *   a.getH() === 12;
     * </pre>
     */
    int getH();


    /**
     * @return ajan minuutit
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,50);
     *   a.getM() === 50;
     * </pre>
     */
    int getM();


    /**
     * @param a aika johon verrataan
     * @return true jos ollaan samanlainen kuin a
     * @example
     * <pre name="test">
     * Aika a1 = new Aika(13,37);
     * Aika a2 = new Aika(13,37);
     * a1.equals(a2) === true;
     * </pre>
     */
    public boolean equals(AikaRajapinta a);
    
    
    /**
     * @return onko oliot samoja
     * @example
     * <pre name="test">
     * Aika a1 = new Aika(13,37);
     * Aika a2 = new Aika(13,37);
     * a1 === a2
     * </pre>
     */
    @Override
    boolean equals(Object o);


    /**
     * @return olio klooni
     * @example
     * <pre name="test">
     * Aika aika = new Aika(13,37);
     * Aika aika2 = aika.clone();
     * aika2.toString() === "13:37";
     * </pre>
     */
    
    AikaRajapinta clone();


    /**
     * @return hajatusarvo
     * Aika sekunteina vuorokauden alusta
     * @example
     * <pre name="test">
     * new Aika(13,37).hashCode() === 49020;
     * </pre>
     */
    @Override
    int hashCode();

}