package kerho;

import java.util.List;

/**
 * CRC sisällöt tähän
 * @author vesal
 * @version 23.2.2021
 */
public class Kerho {

    private Jasenet jasenet = new Jasenet(); 
    private Harrastukset harrastukset = new Harrastukset();
    
    /**
     * Lisätään uusi jäsen
     * @param jasen lisättävä jäsen
     * @throws SailoException  jos lisääminen ei onnistu
     */ 
    public void lisaa(Jasen jasen) throws SailoException {
        jasenet.lisaa(jasen);
    }
    
    
    /**
     * Listään uusi harrastus kerhoon
     * @param har lisättävä harrastus 
     */
    public void lisaa(Harrastus har) {
        harrastukset.lisaa(har);    
    }
  
    
    /**
     * Haetaan kaikki jäsen harrastukset
     * @param jasen jäsen jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Kerho kerho = new Kerho();
     *  Jasen aku1 = new Jasen(), aku2 = new Jasen(), aku3 = new Jasen();
     *  aku1.rekisteroi(); aku2.rekisteroi(); aku3.rekisteroi();
     *  int id1 = aku1.getTunnusNro();
     *  int id2 = aku2.getTunnusNro();
     *  Harrastus pitsi11 = new Harrastus(id1); kerho.lisaa(pitsi11);
     *  Harrastus pitsi12 = new Harrastus(id1); kerho.lisaa(pitsi12);
     *  Harrastus pitsi21 = new Harrastus(id2); kerho.lisaa(pitsi21);
     *  Harrastus pitsi22 = new Harrastus(id2); kerho.lisaa(pitsi22);
     *  Harrastus pitsi23 = new Harrastus(id2); kerho.lisaa(pitsi23);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = kerho.annaHarrastukset(aku3);
     *  loytyneet.size() === 0; 
     *  loytyneet = kerho.annaHarrastukset(aku1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == pitsi21 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }

    
    /**
     * @return jäsenten lkm
     */
    public int getJasenia() {
        return jasenet.getLkm();
    }
    
    
    /**
     * Antaa kerhon i:n jäsenen
     * @param i monesko jäsen (alkaa 0:sta)
     * @return jäsen paikasta i
     */
    public Jasen annaJasen(int i) {
        return jasenet.anna(i);
    }
    
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args)  {
        Kerho kerho = new Kerho();
        
        Jasen aku = new Jasen();
        Jasen aku2 = new Jasen();
        aku.rekisteroi();
        aku.vastaaAkuAnkka();     
        aku2.rekisteroi();
        aku2.vastaaAkuAnkka();    

        try {
            kerho.lisaa(aku);
            kerho.lisaa(aku2);
        } catch (SailoException e) {
            // e.printStackTrace();
            System.err.println(e.getMessage());
        }

        for (int i=0; i<kerho.getJasenia(); i++) {
            Jasen jasen = kerho.annaJasen(i);
            jasen.tulosta(System.out);
        }
    }

 
}
