package dynaaminen;
/**
 * Esimerkki linkitetytstä listasta Java 1.5:n geneerisyyttä
 * ja "autoboxingia" käyttäen.
 * @author Vesa Lappalainen
 * @version 1.0, 16.03.2002
 * @version 1.1, 01.03.2005
 * @param <TYPE> tyyppi, jota talletetaan
 * @example
 * <pre name="test">
 *   LinListaGen<String> sanat = new LinListaGen<String>();
 *   sanat.lisaa("kissa");
 *   sanat.lisaa("koira");
 *   sanat.toString() === " kissa koira";
 * </pre>
 */
@SuppressWarnings({"javadoc"})
public class LinListaGen<TYPE> {

  /**
   * Linkitetyn listan yksi alkio.  Sisältää datan ja viitteen seuraavaan.
   * @param <TYPE> tyyppi jota talletetaan
   */
  public static class Alkio<TYPE> {
    private TYPE sisalto;
    private Alkio<TYPE> seuraava;
    public Alkio(TYPE sisalto) { this.sisalto = sisalto; seuraava = null; }
    public TYPE getSisalto() { return sisalto; }
  }

  private Alkio<TYPE> eka;
  private Alkio<TYPE> viimeinen;

  public LinListaGen() { eka = null; viimeinen = null; }
 // public LinListaGen(int koko) { this(); }

  public void lisaa(TYPE sisalto) {
    Alkio<TYPE> uusi = new Alkio<TYPE>(sisalto);
    if ( eka != null ) viimeinen.seuraava = uusi;
    else eka = uusi;
    viimeinen = uusi;
  }

  @Override
  public String toString() {
    StringBuilder s = new StringBuilder("");
    for (Alkio<TYPE> alkio=eka; alkio != null; alkio = alkio.seuraava )
      s.append(" " + alkio.getSisalto());
    return s.toString();
  }

  public static void main(String[] args) {
    LinListaGen<Integer> luvut = new LinListaGen<Integer>();
    luvut.lisaa(0);
    luvut.lisaa(2);
    luvut.lisaa(99);
    System.out.println(luvut);
  }
}