package fxKerho;

import static fxKerho.TietueDialogController.getFieldId; 
import java.awt.Desktop;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.StringGrid;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import kerho.Harrastus;
import kerho.Jasen;
import kerho.Kerho;
import kerho.SailoException;

/**
 * Luokka kerhon käyttöliittymän tapahtumien hoitamiseksi.
 * 
 * @author vesal
 * @version 31.12.2015
 * @version 5.1.2015
 */
public class KerhoGUIController implements Initializable {

    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;
    @FXML private Label labelVirhe;
    @FXML private ListChooser<Jasen> chooserJasenet; 
    @FXML private ScrollPane panelJasen;
    
    @FXML private GridPane gridJasen;     
    @FXML StringGrid<Harrastus> tableHarrastukset; 
    
    private String kerhonnimi = "kelmit";
    
    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();      
    }

    
    @FXML private void handleHakuehto() {
        String hakukentta = cbKentat.getSelectedText();
        String ehto = hakuehto.getText(); 
        if ( ehto.isEmpty() )
            naytaVirhe(null);
        else
            naytaVirhe("Ei osata vielä hakea " + hakukentta + ": " + ehto);
    }
    
    
    @FXML private void handleTallenna() {
        tallenna();
    }
    
    
    @FXML private void handleAvaa() {
        avaa();
    }
    
    
    @FXML private void handleTulosta() {
        TulostusController.tulosta(null);
    } 

    
    @FXML private void handleLopeta() {
        tallenna();
        Platform.exit();
    } 

    
    @FXML private void handleUusiJasen() {
        uusiJasen();
    }
    
    
    @FXML private void handleMuokkaaJasen() {
        muokkaa(kentta);
    }
    
    
    @FXML private void handlePoistaJasen() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa jäsentä");
    }
    
     
    @FXML private void handleUusiHarrastus() {
        uusiHarrastus();  
    }
    

    @FXML private void handleMuokkaaHarrastus() {
        muokkaaHarrastusta();
    }
    

    @FXML private void handlePoistaHarrastus() {
        Dialogs.showMessageDialog("Ei osata vielä poistaa harrastusta");
    }
    

    @FXML private void handleApua() {
        avustus();
    }
    

    @FXML private void handleTietoja() {
        // Dialogs.showMessageDialog("Ei osata vielä tietoja");
        ModalController.showModal(KerhoGUIController.class.getResource("AboutView.fxml"), "Kerho", null, "");
    }
    

//===========================================================================================    
// Tästä eteenpäin ei käyttöliittymään suoraan liittyvää koodia    
  
    private Kerho kerho;
    private TextField[] edits; 
    private int kentta = 0; 
    private static Harrastus apuharrastus = new Harrastus();
    
    private void alusta() {
        panelJasen.setFitToHeight(true);
        chooserJasenet.clear();
        chooserJasenet.addSelectionListener(e -> naytaJasen());
        edits = TietueDialogController.luoKentat(gridJasen, new Jasen());
        for (TextField edit: edits)   
           if ( edit != null ) {   
               edit.setEditable(false);   
               edit.setOnMouseClicked(e -> { if ( e.getClickCount() > 1 ) muokkaa(getFieldId(edit,kentta)); });  
               edit.focusedProperty().addListener((a,o,n) -> kentta = getFieldId(edit,kentta)); 
           }
        
        // alustetaan harrastustaulukon otsikot 
        int eka = apuharrastus.ekaKentta(); 
        int lkm = apuharrastus.getKenttia(); 
        String[] headings = new String[lkm-eka]; 
        for (int i=0, k=eka; k<lkm; i++, k++) headings[i] = apuharrastus.getKysymys(k); 
        tableHarrastukset.initTable(headings); 
        tableHarrastukset.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY); 
        tableHarrastukset.setEditable(false); 
        tableHarrastukset.setPlaceholder(new Label("Ei vielä harrastuksia")); 
         
        // Tämä on vielä huono, ei automaattisesti muutu jos kenttiä muutetaan. 
        tableHarrastukset.setColumnSortOrderNumber(1); 
        tableHarrastukset.setColumnSortOrderNumber(2); 
        tableHarrastukset.setColumnWidth(1, 60); 
        tableHarrastukset.setColumnWidth(2, 60); 

        
        tableHarrastukset.setEditable(false);
        tableHarrastukset.setOnMouseClicked( e -> { if ( e.getClickCount() > 1 ) muokkaaHarrastusta(); } );
        tableHarrastukset.setOnKeyPressed( e -> {if ( e.getCode() == KeyCode.F2 ) muokkaaHarrastusta();}); 
    }
    
    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    private void setTitle(String title) {
        ModalController.getStage(hakuehto).setTitle(title);
    }
    
    
    /**
     * Alustaa kerhon lukemalla sen valitun nimisestä tiedostosta
     * @param nimi tiedosto josta kerhon tiedot luetaan
     */
    private void lueTiedosto(String nimi) {
        kerhonnimi = nimi;
        setTitle("Kerho - " + kerhonnimi);
        try {
            kerho.lueTiedostosta(nimi);
            hae(0);
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }

    
    /**
     * Kysytään tiedoston nimi ja luetaan se
     * @return true jos onnistui, false jos ei
     */
    public boolean avaa() {
        String uusinimi = KerhonNimiController.kysyNimi(null, kerhonnimi);
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }

    
    /**
     * Tietojen tallennus
     */
    private void tallenna() {
        try {
            kerho.tallenna();
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }
    

    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/2019k/ht/vesal");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    }
    

    private void naytaJasen() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject();
    
        if (jasenKohdalla == null) return;
        
        TietueDialogController.naytaTietue(edits, jasenKohdalla);
        naytaHarrastukset(jasenKohdalla);
    }
    
    
    private void naytaHarrastukset(Jasen jasen) {
        tableHarrastukset.clear();
        if (jasen == null) return;
        List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
        if (harrastukset.size() == 0) return;
        for (Harrastus har: harrastukset) 
            naytaHarrastus(har);
    }
    
    
    private void naytaHarrastus(Harrastus har) {
        int kenttia = har.getKenttia(); 
        String[] rivi = new String[kenttia-har.ekaKentta()]; 
        for (int i=0, k=har.ekaKentta(); k < kenttia; i++, k++) 
            rivi[i] = har.anna(k); 
        tableHarrastukset.add(har,rivi);
    }


    private void muokkaaHarrastusta() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        int r = tableHarrastukset.getRowNr();
        if ( r < 0 ) return; // klikattu ehkä otsikkoriviä
        Harrastus har = tableHarrastukset.getObject();
        if ( har == null ) return;
        int k = tableHarrastukset.getColumnNr()+har.ekaKentta();
        try {
            har = TietueDialogController.kysyTietue(null, har.clone(), k);
            if ( har == null ) return;
            kerho.korvaaTaiLisaa(har); 
            naytaHarrastukset(jasenKohdalla); 
            tableHarrastukset.selectRow(r);  // järjestetään sama rivi takaisin valituksi
        } catch (CloneNotSupportedException  e) { /* clone on tehty */  
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia lisäämisessä: " + e.getMessage());
        }
    }

    
    private void muokkaa(int k)  {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        if (jasenKohdalla == null) return;
        try {
            Jasen jasen = TietueDialogController.kysyTietue(null, jasenKohdalla.clone(), k);
            if (jasen == null) return;
            kerho.korvaaTaiLisaa(jasen);
            hae(jasen.getTunnusNro());
        } catch (CloneNotSupportedException | SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }
    
    
    @SuppressWarnings("unused") // TODO: käytä ja poista varoitus
    private void tulosta(PrintStream os, final Jasen jasen) {
        os.println("----------------------------------------------");
        jasen.tulosta(os);
        os.println("----------------------------------------------");
        List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
        for (Harrastus har: harrastukset) 
            har.tulosta(os);
        os.println("----------------------------------------------");
    }
    
    
    /**
     * Astetaan käytettävä kerho
     * @param kerho jota käytetään
     */
    public void setKerho(Kerho kerho) {
        this.kerho = kerho;
    }

    
    private void hae(int jnro) {
        chooserJasenet.clear();
        
        int index = 0;
        for (int i=0; i < kerho.getJasenia(); i++) {
            Jasen jasen = kerho.annaJasen(i);
            if (jasen.getTunnusNro() == jnro) index = i;
            chooserJasenet.add(""+jasen.getNimi(), jasen);
        }
        chooserJasenet.setSelectedIndex(index);
    }
    
    
    /**
     * Lisätään kerhoon uusi jäsen
     */
    /**
     * Luo uuden jäsenen jota aletaan editoimaan 
     */
    protected void uusiJasen() {
        try {
            Jasen uusi = new Jasen();
            uusi = TietueDialogController.kysyTietue(null, uusi, uusi.ekaKentta()); 
            if ( uusi == null ) return; 
            uusi.rekisteroi(); 
            kerho.lisaa(uusi);
            hae(uusi.getTunnusNro()); 
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
    } 
    
    /** 
     * Tekee uuden tyhjän harrastuksen editointia varten 
     */ 
    private void uusiHarrastus() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        if ( jasenKohdalla == null ) return;

        Harrastus uusi = new Harrastus(jasenKohdalla.getTunnusNro());
        uusi = TietueDialogController.kysyTietue(null, uusi, 0);
        if ( uusi == null ) return;
        uusi.rekisteroi();
        kerho.lisaa(uusi);
        naytaHarrastukset(jasenKohdalla); 
        tableHarrastukset.selectRow(1000);  // järjestetään viimeinen rivi valituksi
    }
    
}
