package olio;

/**
 * Ensimmäinen kunnon olioesimerkki
 * @author Vesa Lappalainen 
 * @version 1.0, 01.02.2003
 * @author Santtu Viitanen 
 * @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);    a1.toString() === "13:10";
 * </pre>
 */
public class Aika {   // Luokka

    private int h = 0, m = 0;  // Attribuutti

    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h, int m) { // Muodostaja
        this.h = h;
        this.m = m;
    }

    
    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", h, m);
    }

    
    /**
     * Lisää aikaan valitun minuuttimäärän
     * @param lisaMin lisättävä minuuttimäärä
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    public void lisaa(int lisaMin) {          // Metodi
        int yht_min = h * 60 + m + lisaMin;
        h = yht_min / 60;
        m = yht_min % 60;
    }

    
    /**
     * Testataan Aika-luokkaa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Aika a1 = new Aika(12, 15);         // Olio
        Aika a2 = new Aika(13, 16);
        Aika a3 = new Aika(14, 25);

        a1.lisaa(55);        System.out.println(a1);
        a2.lisaa(27);        System.out.println(a1);
        a3.lisaa(39);        System.out.println(a1);
    }
    
}
