package valinta;

import fi.jyu.mit.ohj2.IO;
import fi.jyu.mit.ohj2.Mjonot;

/**
 * Luokka äänestyksen käyttöliittymälle
 * @author Vesa Lappalainen
 * @version 1.0, 27.02.2003
 * @version 1.1, 03.02.2003
 */
public class AanestysLiittyma {
    // #import fi.jyu.mit.ohj2.Suuntaaja;

    private final Vaihtoehdot2 valinnat;


    /**
     * Alustetaan sisäinen data-osoitin
     * @param valinnat data jota käyttöliittymä käyttää
     */
    public AanestysLiittyma(Vaihtoehdot2 valinnat) {
        this.valinnat = valinnat;
    }


    /**
     * Kysytään halutaanko jatkaa suoritusta
     * @return jatketaanko (true) vai ei (false)
     * @example
     * <pre name="test">
     *   Suuntaaja.StringInput si = new Suuntaaja.StringInput("");  
     *   Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     *     
     *   si.input("\n");   jatketaan() === true;
     *   si.input("k\n");  jatketaan() === true;
     *   si.input("e\n");  jatketaan() === false;
     *     
     *   si.palauta(); so.palauta();
     * </pre>       
     */
    public static boolean jatketaan() {
        char c;
        System.out.println("Jatketaanko (K/e):");
        c = IO.odota_nappain("KE", 'K', IO.MERKKI_ISOKSI);
        System.out.println("" + c);
        return (c == 'K');
    }


    /**
     * Tulostetaan äänestyksen tämän hetkinen tilanne
     * @example
     * <pre name="test">
     *   Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     *     
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *
     *   AanestysLiittyma liittyma = new AanestysLiittyma(valinnat);
     *   liittyma.tulostaAanet();
     *   String tulos =
     *     "Tilanne: \n" +
     *     "Tyttö     : 0\n" +
     *     "Poika     : 0\n";
     *  
     *   so.ero(tulos) === null;
     *   so.palauta();
     * </pre>       
     */
    public void tulostaAanet() {
        System.out.println("Tilanne: ");
        for (int i = 0; i < valinnat.getLkm(); i++) {
            Valinta valinta = valinnat.anna(i);
            System.out.println(Mjonot.fmt(valinta.getTeksti(), -10) + ": " + valinta.getMaara());
        }
    }


    /**
     * Tulostetaan menu ja odotetaan niinkauan kunnes käyttäjä painaa
     * menussa sallittua kirjainta.
     * @return käyttäjän painama kirjain.
     * @example
     * <pre name="test">
     *   Suuntaaja.StringInput si = new Suuntaaja.StringInput("");  
     *   Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     *     
     *     
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *
     *   AanestysLiittyma liittyma = new AanestysLiittyma(valinnat);
     *   si.input("T\n");
     *   liittyma.tulostaMenuJaOdota() === 'T';
     *  
     *   String tulos =
     *     "  T = Tyttö\n" +
     *     "  P = Poika\n" +
     *     "  :";
     *   so.ero(tulos) === null;
     *   si.palauta(); so.palauta();
     * </pre>       
     */
    public char tulostaMenuJaOdota() {
        if ( valinnat.getLkm() <= 0 ) return ' ';
        String sallitut = valinnat.getKirjaimet();

        char oletus = valinnat.anna(0).getKirjain();

        for (int i = 0; i < valinnat.getLkm(); i++) {
            Valinta valinta = valinnat.anna(i);
            System.out.println("  " + valinta.getTulostus());
        }

        System.out.print("  :");
        return IO.odota_nappain(sallitut, oletus, IO.MERKKI_ISOKSI);
    }


    /**
     * Pyöritetaan äänestyssilmukkaa kunnes käyttäjä ei halua enää jatkaa.
     * @param alkuteksti teksti joka tulostuu ennen menua
     * @param toteamus teksti joka tulostuu "äänestyksen" jälkeen ja jonka
     *        perään tulostetaan valittu vaihtoehto.
     *        
     * @example
     * <pre name="test">
     *   Suuntaaja.StringInput si = new Suuntaaja.StringInput("");  
     *   Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     *     
     *     
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *   valinnat.lisaa(new Valinta('N',"Nainen"));
     *   valinnat.lisaa(new Valinta('M',"Mies"));
     *
     *   AanestysLiittyma liittyma = new AanestysLiittyma(valinnat);
     *   si.input("t\nk\np\ne\n");
     *   liittyma.aanesta("Mitä sukupuolta olet","Olet siis");
     *  
     *   String tulos =
     *     "Mitä sukupuolta olet\n" +
     *     "\n" +
     *     "Valitse:\n" +
     *     "  T = Tyttö\n" +
     *     "  P = Poika\n" +
     *     "  N = Nainen\n" +
     *     "  M = Mies\n" +
     *     "  :\n" +
     *     "Olet siis Tyttö\n" +
     *     "Tilanne: \n" +
     *     "Tyttö     : 1\n" +
     *     "Poika     : 0\n" +
     *     "Nainen    : 0\n" +
     *     "Mies      : 0\n" +
     *     "Jatketaanko (K/e):\n" +
     *     "K\n" +
     *     "Mitä sukupuolta olet\n" +
     *     "\n" +
     *     "Valitse:\n" +
     *     "  T = Tyttö\n" +
     *     "  P = Poika\n" +
     *     "  N = Nainen\n" +
     *     "  M = Mies\n" +
     *     "  :\n" +
     *     "Olet siis Poika\n" +
     *     "Tilanne: \n" +
     *     "Tyttö     : 1\n" +
     *     "Poika     : 1\n" +
     *     "Nainen    : 0\n" +
     *     "Mies      : 0\n" +
     *     "Jatketaanko (K/e):\n" +
     *     "E\n";
     *   so.ero(tulos) === null;
     *   si.palauta(); so.palauta();
     * </pre>       
     */
    public void aanesta(String alkuteksti, String toteamus) {
        do {
            System.out.println(alkuteksti);
            System.out.println("");
            System.out.println("Valitse:");

            char painettu = tulostaMenuJaOdota();

            Valinta val = valinnat.etsi(painettu);
            if ( val == null ) return; // tähän jotakin fiksumpaa

            System.out.println("");
            System.out.println(toteamus + " " + val.getTeksti());

            val.lisaa();

            tulostaAanet();
        } while ( jatketaan() );

    }


    /**
     * Käynnistää äänestyksen
     * @param valinnat lista valinnoista, joita voidaan tehdä
     * @param alkuteksti teksti joka näytetään lomakkeen yläotsikkona 
     * @param toteamus teksti joka tulee jokaisen äänestyksen jälkeen.
     */
    public static void aanesta(Vaihtoehdot2 valinnat, String alkuteksti, String toteamus) {
        AanestysLiittyma liittyma = new AanestysLiittyma(valinnat);
        liittyma.aanesta(alkuteksti, toteamus);
        System.out.println("Kiitos vastauksista!  Hei hei!");
    }


    /**
     * Testataan äänestysliitymää
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
        /*
        valinnat.lisaa(new Valinta('K',"Kyllä"));
        valinnat.lisaa(new Valinta('E',"Ei"));
        valinnat.lisaa(new Valinta('O',"EOS"));
        AanestysLiittyma.aanesta(valinnat,"Onko ohjelmointi kivaa","Äänestit siis");
        */
        valinnat.lisaa(new Valinta('T', "Tyttö"));
        valinnat.lisaa(new Valinta('P', "Poika"));
        valinnat.lisaa(new Valinta('N', "Nainen"));
        valinnat.lisaa(new Valinta('M', "Mies"));
        AanestysLiittyma.aanesta(valinnat, "Mitä sukupuolta olet", "Olet siis");
    }
}