/**
 * @file vaihdamain.cpp
 * Esimerkki aliohjelmasta joka vaihtaa alkioita
 *
 * @author Vesa Lappalainen
 * @version 27.4.2003
 */

#include <iostream>
using namespace std;


/**
 * Vaihdetaan kaksi lukua keskenn
 * @param a ensimminen vaihdettava
 * @param toinen vaihdettava
 * @code
 * <pre name="test">
 *   int a = 3;
 *   int b = 4;
 *   vaihda(&a,&b);
 *   b === 3;
 *   a === 4;
 * </pre>
 * @endcode
 */
void vaihda(int *a, int *b)
{
	int t = *a;
	*a = *b;
	*b = t;
}


/**
 * Vaihdetaan kaksi lukua keskenn
 * @param a ensimminen vaihdettava
 * @param toinen vaihdettava
 * @code
 * <pre name="test">
 *   int a = 3;
 *   int b = 4;
 *   vaihda(a,b);
 *   b === 3;
 *   a === 4;
 * </pre>
 * @endcode
 */
void vaihda(int &a, int &b)
{
    int t = a;
    a = b;
    b = t;
}


void tulosta(int h, int m)
{
  cout << h << ":" << m << endl;
}


int vaihdamainTestOrigMain(void)
{
  int h=12,m=15;
  tulosta(h,m);
  vaihda(&h,&m);
  tulosta(h,m);
  return 0;
}
#include <iostream>
#include <cstring>
namespace comtestCppNameSpace {
//#define assertEquals(s,e,g) { if ( comtestCppNameSpace::assertEqualsImpl((s),(e),(g)) ) return; }
//#define assertEqualsDelta(s,e,g,delta) { if ( comtestCppNameSpace::assertEqualsImpl((s),(e),(g),(delta)) ) return; }
#define assertEquals comtestCppNameSpace::assertEqualsImpl

template <class T1>
class assertion_traits {
  public: static bool equal( const T1& x, const T1& y ) {
    return x == y;
  }
};


template<>
class assertion_traits<const char *> {
  public: static bool equal( const char *x,const char *y) {
    if ( x == 0 && y == 0 ) return true;
    if ( x == 0 || y == 0 ) return false;
    return std::strcmp(x,y) == 0;
  }
};

template<>
class assertion_traits<char *> {
  public: static bool equal( const char *x,char *y) {
    if ( x == 0 && y == 0 ) return true;
    if ( x == 0 || y == 0 ) return false;
    return std::strcmp(x,y) == 0;
  }
};

template <class T1>
int assertEqualsImpl(const char *msg,const T1 &expected, const T1 &actual) {
if ( assertion_traits<T1>::equal(expected,actual) ) return 0;
std::cout << msg << ": expected [" << expected << "] actual [" << actual << "]" << std::endl;
throw int(1);
}

template <class T1>
int assertEqualsImpl(const char *msg,const char  *expected, const T1 &actual) {
if ( assertion_traits<T1>::equal(expected,actual) ) return 0;
if ( expected == 0 ) std::cout << msg << ": expected [null] actual [" << actual << "]" << std::endl;
else if ( &actual == 0 ) std::cout << msg << ": expected [" << expected << "] actual [null]" << std::endl;
else std::cout << msg << ": expected [" << expected << "] actual [" << actual << "]" << std::endl;
throw int(1);
}

int assertEqualsImpl(const char *msg,int expected, int actual) {
if ( expected == actual ) return 0;
std::cout << msg << ": expected [" << expected << "] actual [" << actual << "]" << std::endl;
throw int(1);
}

int assertEqualsImpl(const char *msg,long expected, long actual) {
if ( expected == actual ) return 0;
std::cout << msg << ": expected [" << expected << "] actual [" << actual << "]" << std::endl;
throw int(1);
}


int assertEqualsImpl(const char *msg,double expected, double actual, double delta) {
if ( -delta <= (actual-expected) && (actual-expected) <= delta ) return 0;
std::cout << msg << ": expected [" << expected << "] actual [" << actual << "]" << std::endl;
throw int(1);
}

}

/**
 * Test class made by ComTest
 * @version 2014.04.22 20:29:48 // Generated by ComTest
 *
 */
class vaihdamainTest {


  // Generated by ComTest BEGIN
  /** testVaihda18 */
  public: void testVaihda18() {    // vaihdamain: 18
    int a = 3; 
    int b = 4; 
    vaihda(&a,&b); 
    assertEquals("From: vaihdamain line: 22", 3, b); 
    assertEquals("From: vaihdamain line: 23", 4, a); 
  } // Generated by ComTest END


  // Generated by ComTest BEGIN
  /** testVaihda40 */
  public: void testVaihda40() {    // vaihdamain: 40
    int a = 3; 
    int b = 4; 
    vaihda(a,b); 
    assertEquals("From: vaihdamain line: 44", 3, b); 
    assertEquals("From: vaihdamain line: 45", 4, a); 
  } // Generated by ComTest END

  int runner(void) {
      int errors = 0;
      try { testVaihda18(); } catch (...) { errors++; }
      try { testVaihda40(); } catch (...) { errors++; }
      if ( errors == 0 ) std::cout << "ok" << std::endl;
      return errors;
  }

};

int main(void) {
    vaihdamainTest t; t.runner();
    return 0;
}
